﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;

public class HoppingManager : MonoBehaviour 
{
	public GameObject platformPrefab;
	public GameObject player;
	public GameObject camera;
	public SpriteButton jumpButton;
	public List<PlatformManager> platforms;
	public PlatformManager lastPlatform;
	public float platformDistance;
	public int highScore = 0;
	public float score = 0;
	public float platformSizing = 500;
	public float platformSpeeding = 50;
	public int waterCounter
	{
		get
		{
			return _waterCounter;
		}
		set
		{
			_waterCounter = value;
			if(_waterCounter >= 20)
				Application.LoadLevel("Instructions_Min2");
			else
			{
				counterDisplay.text = waterCounter.ToString() + "/20";
			}
		}
	}
	private int _waterCounter;
	public TextMesh counterDisplay;

	private bool isGameOver = false;

	public void SpawnNewPlatform()
	{
		GameObject platformGO = Instantiate(platformPrefab, new Vector3(transform.position.x, lastPlatform.transform.position.y + platformDistance, transform.position.z), Quaternion.identity) as GameObject;
		PlatformManager platform = platformGO.GetComponent<PlatformManager> ();

		platform.GetComponent<PlatformManager>().speed += (float)(score / platformSpeeding);
		platform.transform.localScale = new Vector3( (platform.transform.localScale.x - score / platformSizing), platform.transform.localScale.y, platform.transform.localScale.z);

		lastPlatform = platform;
	}

	public void AddPlatform(PlatformManager platform)
	{
		platforms.Add(platform);
	}

	public void RemovePlatform(PlatformManager platform)
	{
		platforms.Remove (platform);
	}

	public void StartPlatforms()
	{
		foreach(PlatformManager platform in platforms)
			if (platform.untouched)
				platform.StartPlatform();
	}

	public void Update()
	{
		if (isGameOver)
			return;

		if(camera.transform.position.y - player.transform.position.y > 10)
		{
			Application.LoadLevel ("GameOver_Min1");
		}
	}



	public void GameOver()
	{
		foreach(PlatformManager platform in platforms)
			platform.Stop();

		isGameOver = true;
		camera.GetComponent<CameraFollow>().scrolling = false;
	}

	public void Start ()
	{
		foreach (var platform in platforms)
		{
			if (lastPlatform == null)
			{
				lastPlatform = platform;
				continue;
			}

			platform.transform.position = new Vector3(
				platform.transform.position.x,
				lastPlatform.transform.position.y + platformDistance,
				platform.transform.position.z
			);

			lastPlatform = platform;
		}
		StartPlatforms ();
	}
}
