﻿using UnityEngine;
using System.Collections;

public class PlatformManager : MonoBehaviour 
{
	public GameObject myRigidPlatform;
	public GameObject player;
	public GameObject screenCenter;
	public GameObject checker;
	public HoppingManager hoppingManager;
	public float leftLimit = -1.8f;
	public float rightLimit = 1.8f;
	public float scrolling = 0.01f;
	public float scrollingMultiplier = 1;
	private Vector3 leftPosition;
	private Vector3 rightPosition;
	public float speed = 2f;
	public platformDirection myDirection;

	public bool untouched = true;
	public bool standing = false;

	public enum platformDirection
	{
//		IDLE,
		LEFT,
		RIGHT
	}

	public void Start()
	{
		player = GameObject.FindGameObjectWithTag("Player");
		hoppingManager = GameObject.FindGameObjectWithTag("hoppingManager").GetComponent<HoppingManager>();
		screenCenter = hoppingManager.camera;
		leftPosition = new Vector3(leftLimit, transform.position.y, transform.position.z);
		rightPosition = new Vector3(rightLimit, transform.position.y, transform.position.z);


		StartPlatform();
	}

	public void StartPlatform()
	{
		if(!standing)
		{
			myDirection = ((platformDirection[]) System.Enum.GetValues(typeof(platformDirection))).PickRandomOne();
	
//			myDirection = platformDirection.IDLE;

			switch(myDirection)
			{
		//		case platformDirection.IDLE:
		//		{
		//			transform.position = new Vector3(Random.Range(leftLimit, rightLimit), transform.position.y, transform.position.z);
		//			break;
		//	
				
				case platformDirection.LEFT:
				{
					leftPosition = new Vector3(leftLimit, transform.position.y, transform.position.z);
					transform.position = new Vector3(Random.Range(leftLimit, rightLimit), transform.position.y, transform.position.z);
					ToLeft();
					break;
				}
					
				case platformDirection.RIGHT:
				{
					rightPosition = new Vector3(rightLimit, transform.position.y, transform.position.z);
					transform.position = new Vector3(Random.Range(leftLimit, rightLimit), transform.position.y, transform.position.z);
					ToRight ();
					break;
				}
			}
		}
	}

	public void ActivateRigidPlatform()
	{
		//if(checker != null)
		//	checker.SetActive(true);

		myRigidPlatform.SetActive(true);
	}

	public void DeactivateRigidPlatform()
	{
		//if(checker != null)
		//	checker.SetActive(false);

		myRigidPlatform.SetActive(false);
	}

	public void ToLeft()
	{
		iTween.MoveTo(gameObject, iTween.Hash (
			"x", leftPosition.x,
			"speed", speed,
			"easeType", iTween.EaseType.linear,
			"oncomplete", "ToRight"
			));
		myDirection = platformDirection.LEFT;
	}

	public void ToRight()
	{
		iTween.MoveTo(gameObject, iTween.Hash (
			"x", rightPosition.x,
			"speed", speed,
			"easeType", iTween.EaseType.linear,
			"oncomplete", "ToLeft"
			));
		myDirection = platformDirection.RIGHT;
	}

	public void Stop()
	{
		iTween.Stop (gameObject);
	}

	public void Update()
	{
		if((screenCenter.transform.position.y - transform.position.y) >= 5.4f)
		{
			if(gameObject.GetComponentInChildren<TupaManager>() == null)
			{
//				float y = (transform.position.y + 8.4f);
				hoppingManager.RemovePlatform(this);
				hoppingManager.SpawnNewPlatform();
				Destroy (gameObject);
			}
		}

		scrolling += Time.deltaTime * scrollingMultiplier;
//		Vector3 scrollingPosition = new Vector3 ( 
//		                                 transform.position.x, 
//		                                 transform.position.y - scrolling, 
//		                                 transform.position.z);

		if((player.transform.parent == null) && player.transform.position.y < transform.position.y)
			DeactivateRigidPlatform();
		if(player.transform.position.y > transform.position.y + 1 && player.GetComponent<TupaManager>().jumping)
			ActivateRigidPlatform();
	}
}
