using UnityEngine;
using System.Collections;

public class SpriteButton: MonoBehaviour 
{
	public Camera viewCamera;

    public Sprite buttonUpSprite;
    public Sprite buttonHoverSprite;
    public Sprite buttonDownSprite;    
    public Sprite buttonPressedSprite;

	public bool isMuted = false;

	public bool checkIsBusy = false;
	
	public GameObject targetObject = null;
	public string messageName = "";
	

    public bool sendGameObject = false;


	SpriteRenderer sprite;

	bool buttonDown = false;
	
	public float targetScale = 1.1f;
	public float scaleTime = 0.05f;
	public float pressedWaitTime = 0.3f;


    public void ClickTest()
    { 
        //Debug.Log("pressed");
    }

	void OnEnable()
	{
		buttonDown = false;	
	}
	
	// Use this for initialization
	void Start () 
	{        
		if (viewCamera == null)
		{
			// Find a camera parent 
            Transform node = transform;
            while (node && node.camera == null)
            {
                node = node.parent;
            }
            if (node && node.camera != null) 
			{
				viewCamera = node.camera;
			}
						
			// ...otherwise, use the main camera
			if (viewCamera == null)
			{
				viewCamera = Camera.main;
			}
		}

        sprite = GetComponent<SpriteRenderer>();
				
		if (collider == null)
		{
            BoxCollider newCollider = gameObject.AddComponent<BoxCollider>();
			Vector3 colliderSize = newCollider.size;
			colliderSize.z = 0.2f;
			newCollider.size = colliderSize;
		}

	}
	
	IEnumerator coScale(Vector3 defaultScale, float startScale, float endScale)
    {
		float t0 = Time.realtimeSinceStartup;
		
		Vector3 scale = defaultScale;
		float s = 0.0f;
		while (s < scaleTime)
		{
			float t = Mathf.Clamp01(s / scaleTime);
			float scl = Mathf.Lerp(startScale, endScale, t);
			scale = defaultScale * scl;
			transform.localScale = scale;
			
			yield return 0;
			s = (Time.realtimeSinceStartup - t0);
		}
		
		transform.localScale = defaultScale * endScale;
    }
	
	IEnumerator LocalWaitForSeconds(float seconds)
	{
		float t0 = Time.realtimeSinceStartup;
		float s = 0.0f;
		while (s < seconds)
		{
			yield return 0;
			s = (Time.realtimeSinceStartup - t0);
		}
	}
	
	IEnumerator coHandleButtonPress(int fingerId)
	{
		buttonDown = true; // inhibit processing in Update()
		bool buttonPressed = true; // the button is currently being pressed
		
		Vector3 defaultScale = transform.localScale;
		
		// Button has been pressed for the first time, cursor/finger is still on it
		if (targetScale != 1.0f)
		{
			// Only do this when the scale is actually enabled, to save one frame of latency when not needed
			yield return StartCoroutine( coScale(defaultScale, 1.0f, targetScale) );
		}

        if (buttonDownSprite)
        {
            sprite.sprite = buttonDownSprite;
        }
		
		while (true)
		{
			Vector3 cursorPosition = Vector3.zero;
			bool cursorActive = true;

			// slightly akward arrangement to keep exact backwards compatibility
#if !UNITY_FLASH
			if (fingerId != -1)
			{
				bool found = false;
				for (int i = 0; i < Input.touchCount; ++i)
				{
					Touch touch = Input.GetTouch(i);
					if (touch.fingerId == fingerId)
					{
						if (touch.phase == TouchPhase.Ended || touch.phase == TouchPhase.Canceled)
							break; // treat as not found
						cursorPosition = touch.position;
						found = true;
					}
				}

				if (!found) cursorActive = false;
			} 			
			else
#endif
			{
				if (!Input.GetMouseButton(0))
					cursorActive = false;
				cursorPosition = Input.mousePosition;
			}

			// user is no longer pressing mouse or no longer touching button
			if (!cursorActive) 
				break; 

            Ray ray = viewCamera.ScreenPointToRay(cursorPosition);

            RaycastHit hitInfo;
			bool colliderHit = collider.Raycast(ray, out hitInfo, Mathf.Infinity);
            if (buttonPressed && !colliderHit)
			{
				if (targetScale != 1.0f)
				{
					// Finger is still on screen / button is still down, but the cursor has left the bounds of the button
					yield return StartCoroutine( coScale(defaultScale, targetScale, 1.0f) );
				}
				if (buttonUpSprite)
                   sprite.sprite = buttonUpSprite;								

				buttonPressed = false;
			}
			else if (!buttonPressed & colliderHit)
			{
				if (targetScale != 1.0f)
				{
					// Cursor had left the bounds before, but now has come back in
					yield return StartCoroutine( coScale(defaultScale, 1.0f, targetScale) );
				}

                if (buttonDownSprite)
                {
                    sprite.sprite = buttonDownSprite;
                }
								
				buttonPressed = true;
			}
						
			yield return 0;
		}

		if (buttonPressed)
		{
			if (targetScale != 1.0f)
			{
				// Handle case when cursor was in bounds when the button was released / finger lifted
				yield return StartCoroutine( coScale(defaultScale, targetScale, 1.0f) );
			}

			if (buttonPressedSprite)
                sprite.sprite = buttonPressedSprite;
												
			if (targetObject)
			{																																						
                if (sendGameObject)
				    targetObject.SendMessage(messageName,gameObject);
                else
                    targetObject.SendMessage(messageName);
			}

			// Button may have been deactivated in ButtonPressed / Up event
			// Don't wait in that case
#if UNITY_3_0 || UNITY_3_1 || UNITY_3_2 || UNITY_3_3 || UNITY_3_4 || UNITY_3_5 || UNITY_3_6 || UNITY_3_7 || UNITY_3_8 || UNITY_3_9
			if (gameObject.active)
#else
			if (gameObject.activeInHierarchy)
#endif
			{
				yield return StartCoroutine(LocalWaitForSeconds(pressedWaitTime));
			}

            if (!buttonHoverSprite)
            {
                if (buttonUpSprite)
                    sprite.sprite = buttonUpSprite;
            }
		}
		
		buttonDown = false;
	}

	// Update is called once per frame
	void Update ()
	{
		if (buttonDown) // only need to process if button isn't down
			return;

        Ray ray = viewCamera.ScreenPointToRay(Input.mousePosition);
        RaycastHit hitInfo;
            
        if (Input.GetMouseButtonDown(0))
        {
            if (collider.Raycast(ray, out hitInfo, 1.0e8f))
            {
                if (!Physics.Raycast(ray, hitInfo.distance - 0.01f))
                    StartCoroutine(coHandleButtonPress(-1));
            }
        }
        else
        {
            if (buttonHoverSprite)
            {
                if (collider.Raycast(ray, out hitInfo, 1.0e8f))
                {
                    if (!Physics.Raycast(ray, hitInfo.distance - 0.01f))
                        sprite.sprite = buttonHoverSprite;
                }
                else
                {
                    sprite.sprite = buttonUpSprite;
                }
            }
        }
	}
}
